/*
	GTRH1.C
	gdi32 wrapper for SIORI.EXE (Triangle Heart)
*/

#include "gtrh1.h"

static LONG					dib_width = 640;
static LONG					dib_height = 960;
static WORD					dib_bits = 24;

static BITMAPINFOHEADER		*pdibhdr;
static BYTE					*pdib;

long
calclinebytes(long w, long bits)
{
	long	l;
	
	if (bits == 15) bits = 16;
	l = (w * bits + 7) >> 3;
	l = (l + 3) & (~(3UL));
	return l;
}


MY_EXPORT
HGDIOBJ MYEXPORT
expGetStockObject(int objtype)
{
	HGDIOBJ		h;
	
	h = GetStockObject(objtype);
	return h;
}


MY_EXPORT
int MYEXPORT
expStretchDIBits(
	HDC hdc,
	int dest_x, int dest_y, int dest_w, int dest_h,
	int src_x, int src_y, int src_w, int src_h,
	CONST VOID *pDibEntity,
	CONST BITMAPINFO *pDibInfo,
	UINT usage,
	DWORD rop)
{
	static long	counter;
	int		result;
	LPBITMAPINFOHEADER	pbi = (LPBITMAPINFOHEADER)pDibInfo;
	
	if (pbi->biHeight >= 0) {
		result = StretchDIBits(hdc, dest_x, dest_y, dest_w, dest_h, src_x, src_y, src_w, src_h, pDibEntity, pDibInfo, usage, rop);
	}
	else {
		int		i;
		long	dibw, dibh, cbline;
		long	cboff, cbcopy;
		
		dibw = pbi->biWidth;
		dibh = -(pbi->biHeight);
		
		if (dibw > pdibhdr->biWidth || dibh > pdibhdr->biHeight) {
			char	s[256];
			wsprintf(s, "%dx%d %dbits\rȂɑ傫́AȂc",
				dibw, dibh, pbi->biBitCount);
			MessageBox(NULL, s, "GTRH1 error", MB_ICONHAND | MB_OK);
			result = 0;
		}
		else {
			pdibhdr->biWidth = dibw;
			pdibhdr->biHeight = dibh;
			pdibhdr->biBitCount = pbi->biBitCount;
			cbline = calclinebytes(pdibhdr->biWidth, pdibhdr->biBitCount);
			cboff = src_x * 3;
			cbcopy = src_w;
			if (src_x + src_w > pdibhdr->biWidth) {
				if (src_x >= pdibhdr->biWidth) cbcopy = 0;
				else cbcopy = pdibhdr->biWidth - src_x;
			}
			cbcopy *= 3;
			
			for(i = src_y; i < src_y + src_h; i++) {
				if (i >= pdibhdr->biHeight) continue;
				memcpy(pdib + cbline * (pdibhdr->biHeight - i - 1) + cboff,
					(BYTE *)(pDibEntity) + cbline * i + cboff,
					cbcopy);
			}
			result = StretchDIBits(hdc, dest_x, dest_y, dest_w, dest_h, src_x, src_y, src_w, src_h, pdib, (LPBITMAPINFO)pdibhdr, usage, rop);
		}
		
	}
	
#ifdef DISP_STATUS
	{
		char	s[256];
		
		wsprintf(s, "%d dib %dx%d %dbits : (%d,%d)-(%d,%d) -> (%d,%d) ",
			counter++,
			pbi->biWidth, pbi->biHeight, pbi->biBitCount,
			src_x, src_y, src_x + src_w - 1, src_y + src_h - 1,
			dest_x, dest_y, dest_x + dest_w - 1, dest_y + dest_h - 1
		);
		ExtTextOut(hdc, 0, 0, 0, NULL, s, lstrlen(s), NULL);
	}
#endif

	return result;
}


BOOL
freeDib()
{
	if (!pdibhdr) return TRUE;
	
	VirtualFree(pdibhdr, 0, MEM_RELEASE);
	pdibhdr = NULL;
	pdib = NULL;
	return TRUE;
}


BOOL
allocDib(long width, long height, WORD bits)
{
	BYTE	*p;
	long	cb;
	
	cb = calclinebytes(dib_width, dib_bits);
	p = (BYTE *)VirtualAlloc(NULL, cb * dib_height + sizeof(BITMAPINFOHEADER), MEM_COMMIT, PAGE_READWRITE);
	
	if (!p) return FALSE;
	pdibhdr = (LPBITMAPINFOHEADER)p;
	pdib = p + sizeof(BITMAPINFOHEADER);
	ZeroMemory(p, sizeof(BITMAPINFOHEADER));
	pdibhdr->biSize = sizeof(BITMAPINFOHEADER);
	pdibhdr->biWidth = width;
	pdibhdr->biHeight = height;
	pdibhdr->biPlanes = 1;
	pdibhdr->biBitCount = bits;
	
	return TRUE;
}


BOOL
OnDLLInit()
{
	return allocDib(dib_width, dib_height, dib_bits);
}


BOOL
OnDLLTerm()
{
	return freeDib();
}



int APIENTRY
#ifdef __WATCOMC__
LibMain
#else
DllMain
#endif
(HINSTANCE hi, DWORD fdwReason, LPVOID reserved)
{
	int		rc;
	
	switch(fdwReason) {
		case DLL_PROCESS_ATTACH:
			rc = OnDLLInit();
			break;
		case DLL_PROCESS_DETACH:
			rc = OnDLLTerm();
		default:
			rc = TRUE;
	}
	return rc;
}
